@@ -4,7 +4,7 @@
  * Copyright (C) 2002,2004 Alastair McKinstry, <mckinstry@debian.org>
  * Released under the GPL
  *
- * $Id: at-kbd.c 23733 2004-11-09 13:59:28Z fjpop-guest $
+ * $Id: at-kbd.c 33905 2006-01-09 18:50:16Z smarenka $
  */
 
 #include "config.h"
@@ -21,12 +21,6 @@ kbd_t *at_kbd_get (kbd_t *keyboards, const char *subarch)
 {
 	kbd_t *k = NULL;
 
-#if defined(__m68k__)
-	// On m68k, only mvme, bvme, and q40 have PC-style keyboards
-	if ((strstr(subarch, "vme") == NULL) &&
-	    (strstr(subarch, "q40") == NULL))
-		return keyboards;
-#endif
 #if defined(__powerpc__)
 	// On powerpc, only prep and chrp machines have pc keyboards
 	if ((strstr(subarch, "prep") == NULL) &&
@@ -55,9 +49,9 @@ kbd_t *at_kbd_get (kbd_t *keyboards, const char *subarch)
 	 *
 	 */
 	if (check_dir ("/proc/bus/input")) {
-		if ((grep ("/proc/bus/input/devices","AT Set ") == 0) ||
-		    (grep ("/proc/bus/input/devices","AT Translated Set") == 0) ||
-		    (grep ("/proc/bus/input/devices","AT Raw Set") ==0))
+		if ((grep ("/proc/bus/input/devices","AT Set ") > 0) ||
+		    (grep ("/proc/bus/input/devices","AT Translated Set") > 0) ||
+		    (grep ("/proc/bus/input/devices","AT Raw Set") > 0))
 			k->present = TRUE;
 		else
 			k->present = FALSE;
